==> apache-asterixdb-0.9.3-source-release.zip: apache-asterixdb-0.9.3/asterix-algebra/src/main/java/org/apache/asterix/translator/util/PlanTranslationUtil.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.asterix.translator.util;

import java.util.List;

import org.apache.asterix.lang.common.util.FunctionUtil;
import org.apache.asterix.om.base.AOrderedList;
import org.apache.asterix.om.base.AString;
import org.apache.asterix.om.base.IAObject;
import org.apache.asterix.om.constants.AsterixConstantValue;
import org.apache.asterix.om.functions.BuiltinFunctions;
import org.apache.commons.lang3.mutable.Mutable;
import org.apache.commons.lang3.mutable.MutableObject;
import org.apache.hyracks.algebricks.core.algebra.base.ILogicalExpression;
import org.apache.hyracks.algebricks.core.algebra.base.IVariableContext;
import org.apache.hyracks.algebricks.core.algebra.base.LogicalVariable;
import org.apache.hyracks.algebricks.core.algebra.expressions.ConstantExpression;
import org.apache.hyracks.algebricks.core.algebra.expressions.ScalarFunctionCallExpression;
import org.apache.hyracks.algebricks.core.algebra.expressions.VariableReferenceExpression;
import org.apache.hyracks.algebricks.core.algebra.functions.FunctionIdentifier;
import org.apache.hyracks.algebricks.core.algebra.functions.IFunctionInfo;

public class PlanTranslationUtil {
    private static final LogicalVariable DUMMY_VAR = new LogicalVariable(-1);

    public static void prepareMetaKeyAccessExpression(List<String> field, LogicalVariable resVar,
            List<Mutable<ILogicalExpression>> assignExpressions, List<LogicalVariable> vars,
            List<Mutable<ILogicalExpression>> varRefs, IVariableContext context) {
        IAObject value = (field.size() > 1) ? new AOrderedList(field) : new AString(field.get(0));
        ScalarFunctionCallExpression metaKeyFunction = new ScalarFunctionCallExpression(
                FunctionUtil.getFunctionInfo(BuiltinFunctions.META_KEY));
        metaKeyFunction.getArguments()
                .add(new MutableObject<ILogicalExpression>(new VariableReferenceExpression(resVar)));
        metaKeyFunction.getArguments()
                .add(new MutableObject<>(new ConstantExpression(new AsterixConstantValue(value))));
        assignExpressions.add(new MutableObject<ILogicalExpression>(metaKeyFunction));
        LogicalVariable v = context.newVar();
        vars.add(v);
        if (varRefs != null) {
            varRefs.add(new MutableObject<ILogicalExpression>(new VariableReferenceExpression(v)));
        }
    }

    public static void prepareVarAndExpression(List<String> field, LogicalVariable resVar, List<LogicalVariable> vars,
            List<Mutable<ILogicalExpression>> assignExpressions, List<Mutable<ILogicalExpression>> varRefs,
            IVariableContext context) {
        ScalarFunctionCallExpression f = createFieldAccessExpression(new VariableReferenceExpression(DUMMY_VAR), field);
        f.substituteVar(DUMMY_VAR, resVar);
        assignExpressions.add(new MutableObject<ILogicalExpression>(f));
        LogicalVariable v = context.newVar();
        vars.add(v);
        if (varRefs != null) {
            varRefs.add(new MutableObject<ILogicalExpression>(new VariableReferenceExpression(v)));
        }
    }

    private static ScalarFunctionCallExpression createFieldAccessExpression(ILogicalExpression target,
            List<String> field) {
        FunctionIdentifier functionIdentifier;
        IAObject value;
        if (field.size() > 1) {
            functionIdentifier = BuiltinFunctions.FIELD_ACCESS_NESTED;
            value = new AOrderedList(field);
        } else {
            functionIdentifier = BuiltinFunctions.FIELD_ACCESS_BY_NAME;
            value = new AString(field.get(0));
        }
        IFunctionInfo finfoAccess = FunctionUtil.getFunctionInfo(functionIdentifier);
        return new ScalarFunctionCallExpression(finfoAccess, new MutableObject<>(target),
                new MutableObject<>(new ConstantExpression(new AsterixConstantValue(value))));
    }
}

==> apache-asterixdb-0.9.4-source-release.zip: apache-asterixdb-0.9.4/asterix-algebra/src/main/java/org/apache/asterix/translator/util/PlanTranslationUtil.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.asterix.translator.util;

import java.util.List;

import org.apache.asterix.lang.common.util.FunctionUtil;
import org.apache.asterix.om.base.AOrderedList;
import org.apache.asterix.om.base.AString;
import org.apache.asterix.om.base.IAObject;
import org.apache.asterix.om.constants.AsterixConstantValue;
import org.apache.asterix.om.functions.BuiltinFunctions;
import org.apache.commons.lang3.mutable.Mutable;
import org.apache.commons.lang3.mutable.MutableObject;
import org.apache.hyracks.algebricks.core.algebra.base.ILogicalExpression;
import org.apache.hyracks.algebricks.core.algebra.base.IVariableContext;
import org.apache.hyracks.algebricks.core.algebra.base.LogicalVariable;
import org.apache.hyracks.algebricks.core.algebra.expressions.ConstantExpression;
import org.apache.hyracks.algebricks.core.algebra.expressions.ScalarFunctionCallExpression;
import org.apache.hyracks.algebricks.core.algebra.expressions.VariableReferenceExpression;
import org.apache.hyracks.algebricks.core.algebra.functions.FunctionIdentifier;
import org.apache.hyracks.algebricks.core.algebra.functions.IFunctionInfo;

public class PlanTranslationUtil {
    private static final LogicalVariable DUMMY_VAR = new LogicalVariable(-1);

    public static void prepareMetaKeyAccessExpression(List<String> field, LogicalVariable resVar,
            List<Mutable<ILogicalExpression>> assignExpressions, List<LogicalVariable> vars,
            List<Mutable<ILogicalExpression>> varRefs, IVariableContext context) {
        IAObject value = (field.size() > 1) ? new AOrderedList(field) : new AString(field.get(0));
        ScalarFunctionCallExpression metaKeyFunction =
                new ScalarFunctionCallExpression(FunctionUtil.getFunctionInfo(BuiltinFunctions.META_KEY));
        metaKeyFunction.getArguments()
                .add(new MutableObject<ILogicalExpression>(new VariableReferenceExpression(resVar)));
        metaKeyFunction.getArguments()
                .add(new MutableObject<>(new ConstantExpression(new AsterixConstantValue(value))));
        assignExpressions.add(new MutableObject<ILogicalExpression>(metaKeyFunction));
        LogicalVariable v = context.newVar();
        vars.add(v);
        if (varRefs != null) {
            varRefs.add(new MutableObject<ILogicalExpression>(new VariableReferenceExpression(v)));
        }
    }

    public static void prepareVarAndExpression(List<String> field, LogicalVariable resVar, List<LogicalVariable> vars,
            List<Mutable<ILogicalExpression>> assignExpressions, List<Mutable<ILogicalExpression>> varRefs,
            IVariableContext context) {
        ScalarFunctionCallExpression f = createFieldAccessExpression(new VariableReferenceExpression(DUMMY_VAR), field);
        f.substituteVar(DUMMY_VAR, resVar);
        assignExpressions.add(new MutableObject<ILogicalExpression>(f));
        LogicalVariable v = context.newVar();
        vars.add(v);
        if (varRefs != null) {
            varRefs.add(new MutableObject<ILogicalExpression>(new VariableReferenceExpression(v)));
        }
    }

    private static ScalarFunctionCallExpression createFieldAccessExpression(ILogicalExpression target,
            List<String> field) {
        FunctionIdentifier functionIdentifier;
        IAObject value;
        if (field.size() > 1) {
            functionIdentifier = BuiltinFunctions.FIELD_ACCESS_NESTED;
            value = new AOrderedList(field);
        } else {
            functionIdentifier = BuiltinFunctions.FIELD_ACCESS_BY_NAME;
            value = new AString(field.get(0));
        }
        IFunctionInfo finfoAccess = FunctionUtil.getFunctionInfo(functionIdentifier);
        return new ScalarFunctionCallExpression(finfoAccess, new MutableObject<>(target),
                new MutableObject<>(new ConstantExpression(new AsterixConstantValue(value))));
    }
}
